<?php

namespace Drupal\shy_one_time\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Jaybizzle\CrawlerDetect\CrawlerDetect;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * ShyUserController class extends UserController routines for user routes.
 */
class ShyController extends ControllerBase {

  /**
   * Request check via CrawlerDetect.
   *
   * Check if the url is retrieved by spider, crawler or bot. If this is the
   * case, access is denied, but the url remains valid for normal access by a
   * user.
   *
   * In order to never disclose a reset link via a referrer header this
   * controller must always return a redirect response.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request access.
   * @param int $uid
   *   Unique user ID.
   * @param int $timestamp
   *   Current timestamp.
   * @param string $hash
   *   Hash value as parameter.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response object that may be returned by the controller.
   */
  public function resetPass(Request $request, int $uid, int $timestamp, string $hash): RedirectResponse {

    // When processing the one-time login link, we have to make sure that a
    // spider, bot or crawler does not access this link and thus invalidates it.
    if ((new CrawlerDetect())->isCrawler()) {
      throw new AccessDeniedHttpException();
    }

    $account = $this->currentUser();
    // When processing the one-time login link, we have to make sure that a user
    // isn't already logged in.
    if ($account->isAuthenticated()) {
      // The current user is already logged in.
      if ($account->id() == $uid) {
        user_logout();
        // We need to begin the redirect process again because logging out will
        // destroy the session.
        return $this->redirect(
          'user.reset',
          [
            'uid' => $uid,
            'timestamp' => $timestamp,
            'hash' => $hash,
          ]
        );
      }
      // A different user is already logged in on the computer.
      else {
        /** @var \Drupal\user\UserInterface $reset_link_user */
        if ($reset_link_user = $this->userStorage->load($uid)) {
          $this->messenger()
            ->addWarning($this->t('Another user (%other_user) is already logged into the site on this computer, but you tried to use a one-time link for user %resetting_user. Please <a href=":logout">log out</a> and try using the link again.',
              [
                '%other_user' => $account->getAccountName(),
                '%resetting_user' => $reset_link_user->getAccountName(),
                ':logout' => Url::fromRoute('user.logout')->toString(),
              ]));
        }
        else {
          // Invalid one-time link specifies an unknown user.
          $this->messenger()
            ->addError($this->t('The one-time login link you clicked is invalid.'));
        }
        return $this->redirect('<front>');
      }
    }
  }

}
